import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart' show rootBundle;
import 'package:webview_flutter/webview_flutter.dart';
import 'package:package_info_plus/package_info_plus.dart';

class LongdoMapWidget extends StatefulWidget {
  final String? url;
  final String? apiKey;
  final List<JavascriptChannel> eventName;
  final Object options;

  const LongdoMapWidget({
    Key? key,
    this.url,
    this.apiKey,
    this.eventName = const [],
    this.options = const {},
  }) : super(key: key);

  @override
  LongdoMapState createState() => LongdoMapState();
}

class LongdoMapState extends State<LongdoMapWidget> {
  late WebViewController _controller;

  @override
  void initState() {
    super.initState();
    if (Platform.isAndroid) WebView.platform = AndroidWebView();
  }

  @override
  Widget build(BuildContext context) {
    return WebView(
      javascriptMode: JavascriptMode.unrestricted,
      javascriptChannels: Set.from(widget.eventName +
          [
            JavascriptChannel(
                name: "options",
                onMessageReceived: (_) {
                  _controller
                      .runJavascript("init('${jsonEncode(widget.options)}')");
                })
          ]),
      onWebViewCreated: (WebViewController controller) {
        _controller = controller;
        _load();
      },
    );
  }

  void _load() async {
    // get html
    String content = await rootBundle
        .loadString("packages/longdo_maps_api3_flutter/assets/index.html");
    // set api key
    content = content.replaceFirst("[YOUR_API_URL]",
        widget.url ?? "https://api.longdo.com/map3/?key=${widget.apiKey}");
    content = content.replaceFirst(
        "[YOUR_SECRET_API_KEY]", "${widget.apiKey?.substring(0, 8)}...");
    // set application id
    final bundleId = (await PackageInfo.fromPlatform()).packageName;
    content = content.replaceFirst("[YOUR_BUNDLE_ID]", bundleId);
    // load
    _controller.loadHtmlString(content, baseUrl: "https://$bundleId/");
  }

  Object LongdoStatic(String type, String name) => {
        "\$static": type,
        "name": name,
      };

  Object LongdoObject(
    String type, {
    String? id,
    List<Object>? args,
  }) =>
      {
        "\$object": type,
        "\$id": id ?? DateTime.now().millisecondsSinceEpoch,
        "args": args ?? [],
      };

  Object call(
    method, {
    List<Object>? args,
  }) async {
    String data = await _controller.runJavascriptReturningResult(
        "call('" + method + "','" + json.encode(args ?? []) + "');");
    return json.decode(data);
  }

  Object objectCall(
    obj,
    method, {
    List<Object>? args,
  }) async {
    String data = await _controller.runJavascriptReturningResult(
        "objectCall('" +
            json.encode(obj) +
            "' ,'" +
            method +
            "' ,'" +
            json.encode(args ?? []) +
            "');");
    return json.decode(data);
  }

  run(
    final String javascript, {
    result,
  }) async =>
      result != null
          ? json.decode(
              await _controller.runJavascriptReturningResult(javascript))
          : _controller.runJavascript(javascript);
}
